@@ -17,13 +17,13 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -56,7 +56,10 @@
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/opencl.h"
+#include "magick/option.h"
 #include "magick/pixel-accessor.h"
+#include "magick/profile.h"
+#include "magick/property.h"
 #include "magick/quantum-private.h"
 #include "magick/resource_.h"
 #include "magick/static.h"
@@ -150,6 +153,66 @@ static void InitializeDcrawOpenCL(ExceptionInfo *magick_unused(exception))
 #endif
 #endif
 
+#if defined(MAGICKCORE_RAW_R_DELEGATE)
+static void SetDNGProperties(Image *image,const libraw_data_t *raw_info)
+{
+  char
+    property[MagickPathExtent],
+    timestamp[MagickPathExtent];
+
+  (void) SetImageProperty(image,"dng:make",raw_info->idata.make);
+  (void) SetImageProperty(image,"dng:camera.model.name",raw_info->idata.model);
+  (void) FormatMagickTime(raw_info->other.timestamp,MagickPathExtent,timestamp);
+  (void) SetImageProperty(image,"dng:create.date",timestamp);
+  (void) SetImageProperty(image,"dng:f.number",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+    raw_info->other.iso_speed);
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,18)
+  (void) SetImageProperty(image,"dng:software",raw_info->idata.software);
+  if (*raw_info->shootinginfo.BodySerial != '\0')
+    (void) SetImageProperty(image,"dng:serial.number",
+      raw_info->shootinginfo.BodySerial);
+  (void) FormatLocaleString(property,MagickPathExtent,"1/%0.1f",
+    PerceptibleReciprocal(raw_info->other.shutter));
+  (void) SetImageProperty(image,"dng:exposure.time",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+    raw_info->other.aperture);
+  (void) SetImageProperty(image,"dng:iso.setting",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+    raw_info->lens.EXIF_MaxAp);
+  (void) SetImageProperty(image,"dng:max.aperture.value",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+    raw_info->other.focal_len);
+  (void) SetImageProperty(image,"dng:focal.length",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%f %f %f %f",
+    raw_info->color.cam_mul[0],raw_info->color.cam_mul[2],
+    raw_info->color.cam_mul[1],raw_info->color.cam_mul[3]);
+  (void) SetImageProperty(image,"dng:wb.rb.levels",property);
+  (void) FormatLocaleString(property,MagickPathExtent,
+    "%0.1f-%0.1fmm f/%0.1f-%0.1f",raw_info->lens.makernotes.MinFocal,
+    raw_info->lens.makernotes.MaxFocal,raw_info->lens.makernotes.MaxAp4MinFocal,
+    raw_info->lens.makernotes.MaxAp4MaxFocal);
+  (void) SetImageProperty(image,"dng:lens",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.2f",
+    raw_info->lens.makernotes.LensFStops);
+  (void) SetImageProperty(image,"dng:lens.f.stops",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f mm",
+    raw_info->lens.makernotes.MinFocal);
+  (void) SetImageProperty(image,"dng:min.focal.length",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f mm",
+    raw_info->lens.makernotes.MaxFocal);
+  (void) SetImageProperty(image,"dng:max.focal.length",property);
+  (void) SetImageProperty(image,"dng:max.aperture.at.min.focal",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%0.1f",
+    raw_info->lens.makernotes.MaxAp4MaxFocal);
+  (void) SetImageProperty(image,"dng:max.aperture.at.max.focal",property);
+  (void) FormatLocaleString(property,MagickPathExtent,"%d mm",
+    raw_info->lens.FocalLengthIn35mmFormat);
+  (void) SetImageProperty(image,"dng:focal.length.in.35mm.format",property);
+#endif
+}
+#endif
+
 static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
@@ -162,12 +225,12 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -190,6 +253,9 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
     register ssize_t
       y;
 
+    StringInfo
+      *profile;
+
     unsigned short
       *p;
 
@@ -199,8 +265,11 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
           libraw_strerror(errcode),"`%s'",image->filename);
+        libraw_close(raw_info);
         return(DestroyImageList(image));
       }
+    raw_info->params.use_camera_wb=IsStringTrue(GetImageOption(image_info,
+      "dng:use-camera-wb"));
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_VER) && (_MSC_VER > 1310)
     {
       wchar_t
@@ -217,8 +286,26 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
           libraw_strerror(errcode),"`%s'",image->filename);
+        libraw_close(raw_info);
         return(DestroyImageList(image));
       }
+    image->columns=raw_info->sizes.width;
+    image->rows=raw_info->sizes.height;
+    image->page.width=raw_info->sizes.raw_width;
+    image->page.height=raw_info->sizes.raw_height;
+    image->page.x=raw_info->sizes.left_margin;
+    image->page.y=raw_info->sizes.top_margin;
+    if (image_info->ping != MagickFalse)
+      {
+        libraw_close(raw_info);
+        return(image);
+      }
+    status=SetImageExtent(image,image->columns,image->rows);
+    if (status == MagickFalse)
+      {
+        libraw_close(raw_info);
+        return(image);
+      }
     errcode=libraw_unpack(raw_info);
     if (errcode != LIBRAW_SUCCESS)
       {
@@ -251,7 +338,11 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
     image->columns=raw_image->width;
     image->rows=raw_image->height;
-    image->depth=16;
+    image->depth=raw_image->bits;
+    image->page.width=raw_info->sizes.width;
+    image->page.height=raw_info->sizes.height;
+    image->page.x=raw_info->sizes.left_margin;
+    image->page.y=raw_info->sizes.top_margin;
     status=SetImageExtent(image,image->columns,image->rows);
     if (status == MagickFalse)
       {
@@ -259,12 +350,6 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         libraw_close(raw_info);
         return(DestroyImageList(image));
       }
-    if (image_info->ping != MagickFalse)
-      {
-        libraw_dcraw_clear_mem(raw_image);
-        libraw_close(raw_info);
-        return(image);
-      }
     p=(unsigned short *) raw_image->data;
     for (y=0; y < (ssize_t) image->rows; y++)
     {
@@ -297,6 +382,32 @@ static Image *ReadDNGImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
     libraw_dcraw_clear_mem(raw_image);
+    /*
+      Set DNG image metadata.
+    */
+    if (raw_info->color.profile != NULL)
+      {
+        profile=BlobToStringInfo(raw_info->color.profile,
+          raw_info->color.profile_length);
+        if (profile != (StringInfo *) NULL)
+          {
+            SetImageProfile(image,"ICC",profile);
+            profile=DestroyStringInfo(profile);
+          }
+      }
+#if LIBRAW_COMPILE_CHECK_VERSION_NOTLESS(0,18)
+    if (raw_info->idata.xmpdata != NULL)
+      {
+        profile=BlobToStringInfo(raw_info->idata.xmpdata,
+          raw_info->idata.xmplen);
+        if (profile != (StringInfo *) NULL)
+          {
+            SetImageProfile(image,"XMP",profile);
+            profile=DestroyStringInfo(profile);
+          }
+      }
+#endif
+    SetDNGProperties(image,raw_info);
     libraw_close(raw_info);
     return(image);
   }
@@ -437,7 +548,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Hasselblad CFV/H3D39II");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ARW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -446,7 +557,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Alpha Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("DNG");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -455,7 +566,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Digital Negative");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CR2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -464,7 +575,16 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
+  (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("CR3");
+  entry->decoder=(DecodeImageHandler *) ReadDNGImage;
+  entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->format_type=ExplicitFormatType;
+  entry->description=ConstantString("Canon Digital Camera Raw Image Format");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("CRW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -473,7 +593,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Canon Digital Camera Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("DCR");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -482,7 +602,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ERF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -491,7 +611,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Epson Raw Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("IIQ");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -500,7 +620,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Phase One Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("KDC");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -509,7 +629,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("K25");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -518,7 +638,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Kodak Digital Camera Raw Image Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MEF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -527,7 +647,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Mamiya Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MRW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -536,7 +656,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony (Minolta) Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("NEF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -545,7 +665,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("NRW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -554,7 +674,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Nikon Digital SLR Camera Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ORF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -563,7 +683,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Olympus Digital Camera Raw Image File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PEF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -572,7 +692,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Pentax Electronic File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RAF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -581,7 +701,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Fuji CCD-RAW Graphic File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RAW");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -590,7 +710,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RMF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -599,7 +719,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Raw Media Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("RW2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -608,7 +728,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Panasonic Lumix Raw Image");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SRF");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -617,7 +737,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Raw Format");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("SR2");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -626,7 +746,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->seekable_stream=MagickTrue;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sony Raw Format 2");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("X3F");
   entry->decoder=(DecodeImageHandler *) ReadDNGImage;
@@ -634,7 +754,7 @@ ModuleExport size_t RegisterDNGImage(void)
   entry->blob_support=MagickFalse;
   entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Sigma Camera RAW Picture File");
-  entry->module=ConstantString("DNG");
+  entry->magick_module=ConstantString("DNG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -679,6 +799,7 @@ ModuleExport void UnregisterDNGImage(void)
   (void) UnregisterMagickInfo("ERF");
   (void) UnregisterMagickInfo("DCR");
   (void) UnregisterMagickInfo("CRW");
+  (void) UnregisterMagickInfo("CR3");
   (void) UnregisterMagickInfo("CR2");
   (void) UnregisterMagickInfo("DNG");
   (void) UnregisterMagickInfo("ARW");
