@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -241,7 +241,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case DiskResource:
     {
       limit=resource_info.disk_limit;
-      if ((resource_info.disk+request) > resource_info.disk)
+      if ((resource_info.disk+request) > (MagickOffsetType) resource_info.disk)
         {
           resource_info.disk+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -262,7 +262,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case FileResource:
     {
       limit=resource_info.file_limit;
-      if ((resource_info.file+request) > resource_info.file)
+      if ((resource_info.file+request) > (MagickOffsetType) resource_info.file)
         {
           resource_info.file+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -309,7 +309,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case MapResource:
     {
       limit=resource_info.map_limit;
-      if ((resource_info.map+request) > resource_info.map)
+      if ((resource_info.map+request) > (MagickOffsetType) resource_info.map)
         {
           resource_info.map+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -330,7 +330,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case MemoryResource:
     {
       limit=resource_info.memory_limit;
-      if ((resource_info.memory+request) > resource_info.memory)
+      if ((resource_info.memory+request) > (MagickOffsetType) resource_info.memory)
         {
           resource_info.memory+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -381,7 +381,7 @@ MagickExport MagickBooleanType AcquireMagickResource(const ResourceType type,
     case TimeResource:
     {
       limit=resource_info.time_limit;
-      if ((resource_info.time+request) > resource_info.time)
+      if ((resource_info.time+request) > (MagickOffsetType) resource_info.time)
         {
           resource_info.time+=request;
           if ((limit == MagickResourceInfinity) ||
@@ -1325,19 +1325,6 @@ MagickExport MagickBooleanType RelinquishUniqueFileResource(const char *path)
 %      MagickBooleanType ResourceComponentGenesis(void)
 %
 */
-
-static inline MagickSizeType StringToSizeType(const char *string,
-  const double interval)
-{
-  double
-    value;
-
-  value=SiPrefixToDoubleInterval(string,interval);
-  if (value >= (double) MagickULLConstant(~0))
-    return(MagickULLConstant(~0));
-  return((MagickSizeType) value);
-}
-
 MagickExport MagickBooleanType ResourceComponentGenesis(void)
 {
   char
@@ -1380,6 +1367,9 @@ MagickExport MagickBooleanType ResourceComponentGenesis(void)
   pages=(-1);
 #if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
   pages=(ssize_t) sysconf(_SC_PHYS_PAGES);
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+  pages=pages/2;
+#endif
 #endif
   memory=(MagickSizeType) pages*pagesize;
   if ((pagesize <= 0) || (pages <= 0))
@@ -1591,6 +1581,8 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       else
         resource_info.height_limit=MagickMin(limit,StringToSizeType(value,
           100.0));
+      resource_info.height_limit=MagickMin(resource_info.height_limit,
+        SSIZE_MAX);
       break;
     }
     case ListLengthResource:
@@ -1665,6 +1657,8 @@ MagickExport MagickBooleanType SetMagickResourceLimit(const ResourceType type,
       else
         resource_info.width_limit=MagickMin(limit,StringToSizeType(value,
           100.0));
+      resource_info.width_limit=MagickMin(resource_info.width_limit,
+        SSIZE_MAX);
       break;
     }
     default:
